#!/usr/bin/perl
# dirvish-runall
# 1.3.X series
# Copyright 2005 by the dirvish project
# http://www.dirvish.org
#
# Last Revision   : $Rev: 622 $
# Revision date   : $Date: 2009-01-26 22:27:17 +0100 (Mo, 26 Jan 2009) $
# Last Changed by : $Author: tex $
# Stored as       : $HeadURL: https://secure.id-schulz.info/svn/tex/priv/dirvish_1_3_1/dirvish-runall.pl $

#########################################################################
#                                                         				#
#	Licensed under the Open Software License version 2.0				#
#                                                         				#
#	This program is free software; you can redistribute it				#
#	and/or modify it under the terms of the Open Software				#
#	License, version 2.0 by Lauwrence E. Rosen.							#
#                                                         				#
#	This program is distributed in the hope that it will be				#
#	useful, but WITHOUT ANY WARRANTY; without even the implied			#
#	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR				#
#	PURPOSE.  See the Open Software License for details.				#
#                                                         				#
#########################################################################

#----------------------------------------------------------------------------
# Revision information
my %CodeID = (
    Rev    => '$Rev: 622 $'     ,
    Date   => '$Date: 2009-01-26 22:27:17 +0100 (Mo, 26 Jan 2009) $'    ,
    Author => '$Author: tex $'  ,
    URL    => '$HeadURL: https://secure.id-schulz.info/svn/tex/priv/dirvish_1_3_1/dirvish-runall.pl $' ,
);

$VERSION =   $CodeID{URL};
$VERSION =~  s#^.*dirvish_##;  # strip off the front
$VERSION =~  s#\/.*##;         # strip off the rear after the last /
$VERSION =~  s#[_-]#.#g;       # _ or - to "."

#----------------------------------------------------------------------------
# Modules and includes
#----------------------------------------------------------------------------

use Time::ParseDate;
use POSIX qw(strftime);
use Getopt::Long;
# use strict;
# use warnings;

use Dirvish;     # this will change a lot during development

#----------------------------------------------------------------------------
# Initialisation
#----------------------------------------------------------------------------
$SIG{TERM} = \&sigterm; # handles "kill <PID>"
$SIG{INT} = \&sigterm; # handles Ctrl+C or "kill -2 <PID>"

$Options = {
        version         => sub {
                        print STDERR "dirvish version $VERSION\n";
                        exit(0);
                },
        help            => \&usage,
};

$Options = reset_options( \&usage, @ARGV);   # initialize the %$Options hash
load_master_config();

GetOptions($Options, qw(
        config=s
        quiet no-run|dry-run
        version help
    )) or usage;

$$Options{Dirvish} ||= 'dirvish';

$$Options{'no-run'} and $$Options{quiet} = 0;

$errors = 0;

if(!check_pidfile($$Options{pidfile})) {
	print "$0 already running. Aborting at ".strftime('%H:%M:%S', localtime)." - PID: ".$$."\n";
	exit 255; # Abort
}

if($$Options{'threads'}) {
	use Thread qw(async);
	my @threads = ();
}

for $sched (@{$$Options{Runall}})
{
	($vault, $itime) = split(/\s+/, $sched);
	my $cmd = "$$Options{Dirvish} --vault $vault";
	$itime and $cmd .= qq[ --image-time "$itime"];
	$$Options{quiet}
		or printf "%s %s\n", strftime('%H:%M:%S', localtime), $cmd;
	#$$Options{'no-run'} and next;
	if($$Options{'threads'}) {
		my $t = async {
			$$Options{'no-run'} and $cmd .= " --no-run";
			#print "Thread ".Thread->self->tid()." running cmd: $cmd\n";
		    my $status = system($cmd);
	    	return $status;
	    };
	    push(@threads,$t);
    } else {
    	$status = system($cmd) unless $$Options{'no-run'};
	    $status < 0 || $status / 256 and ++$errors;
    }
}

if($$Options{'threads'}) {
	# Wait for the threads to finish
	for $th (@threads) {
		my ($status, $errors) = $th->join();
		$status < 0 || $status / 256 and ++$errors;
	}
}

remove_pidfile($$Options{pidfile});

$$Options{quiet}
    or printf "%s %s\n", strftime('%H:%M:%S', localtime), 'done';


exit ($errors < 200 ? $errors : 199);
#----------------------------------------------------------------------------
# Subs
#----------------------------------------------------------------------------
sub usage
{
    my $message = shift(@_);

    length($message) and print STDERR $message, "\n\n";

    $! and exit(255); # because getopt seems to send us here for death

    print STDERR <<EOUSAGE;
USAGE
	dirvish-runall OPTIONS
OPTIONS
	--config configfile
	--no-run
	--quiet
	--version
EOUSAGE

	exit 255;
}
# Remove the pidfile on exit
END {
	# remove the pidfile on exit
	remove_pidfile($$Options{pidfile});
}
# Handle SIGTERM (SIG-15)
sub sigterm
{
	print STDERR "Received SIGTERM. Aborting running backup ...";
	# kill childs - kill(TERM, -$$):
	use POSIX;
	my $cnt = kill(SIGTERM, -$$);
	no POSIX;
	print STDERR "Signaled $cnt processes in current processgroup";
	# quit
	exit;
}
#----------------------------------------------------------------------------
# EOF
#----------------------------------------------------------------------------
